﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using parkMonitor.LOG;
using parkMonitor.entity;
using parkMonitor.model;
using System.Configuration;
using System.Threading;
using parkMonitor.server.uiLogServer;
using parkMonitor.tools;

using parkMonitor.Database2;
using MySql.Data.MySqlClient;
using parkMonitor.server.CoreThread;

namespace parkMonitor.server.CoreThread
{
    class CoreThreadTest2 : IEquipments, ICoreThreadDoWorking
    {
        int equipmentStatus_address;//设备总控状态地址
        bool initialized = false;
        /// <summary>
        /// 获取PLC句柄
        /// </summary>
        IEquipments PLC;
        /// <summary>
        /// 获取队列句柄
        /// </summary>
        IEquipments queuingThread;
        /// <summary>
        /// 获取号牌句柄
        /// </summary>
        IEquipments NumMachine;
        bool isClosing;

        string[] equipNames = new string[] { EquipmentName.PLC, EquipmentName.NumMachine, EquipmentName.Web, EquipmentName.Queue };

        public CoreThreadTest2()
        {
            try
            {
                EntityForCore.ins.globalStatus = false;
                equipmentStatus_address = Convert.ToInt32(ConfigurationManager.AppSettings["equipmentStatus_address"]);
            }
            catch (Exception) { }
        }

        public void Start()
        {
            //创建所有实例
            for (int i = 0; i < equipNames.Length; ++i)
            {
                IEquipments equip = EquipmentSimpleFactory.ins.CreateEquipment(equipNames[i]);
            }
            //启动所有实例
            for (int i = 0; i < equipNames.Length; ++i)
            {
                IEquipments equip = EquipmentSimpleFactory.ins.FindEquipment(equipNames[i]);
                equip.Start();
            }
            PLC = EquipmentSimpleFactory.ins.FindEquipment(EquipmentName.PLC);
            queuingThread = EquipmentSimpleFactory.ins.FindEquipment(EquipmentName.Queue);
            NumMachine = EquipmentSimpleFactory.ins.FindEquipment(EquipmentName.NumMachine);
            if (!(EntityForCore.ins.globalStatus = EntityForCore.ins.Init()))
            {
                UILogServer.ins.error("系统初始化异常，请检查配置");
            }
            initialized = true;
            //BeginWorking();
            //监听总状态改变消息
            //由监控线程发出
            AsyncCmdServer.ins.listen(AsyncCmdType.TotalStatusChanged, (AlarmStatus type) =>
            {
                if (type != AlarmStatus.Normal)
                {
                    hangUp();
                }
            });
        }

        public AbstractMessage GetMessage()
        {
            return null;
        }

        public void SetMessage(AbstractMessage message)
        {
        }

        public void Stop()
        {
            for (int i = 0; i < equipNames.Length; ++i)
            {
                IEquipments equip = EquipmentSimpleFactory.ins.FindEquipment(equipNames[i]);
                if (equip != null)
                {
                    equip.Stop();
                }
            }
            AbstractCmd.isClosing = true;//停止运行中指令
            this.isClosing = true;

        }
        /// <summary>
        /// 核心线程工作
        /// </summary>
        public void BeginWorking()
        {
            Timer checkRemoteTimer = new System.Threading.Timer(CheckRemotePool, null, 3600000, 3600000);
            //Timer checkLocalTimer = new System.Threading.Timer(CheckLocalPool, null, 10000, 300000);
            //Timer logTimer = new System.Threading.Timer(displayLog, null, 1000, 30000);
            Object lockObj = new object();
            Command queueCmdRecord = null;
            while (!isClosing)
            {
                lock (lockObj)
                {
                    //判断当前计数是否允许线程创建
                    if (queueCmdRecord != null)
                    {
                        if (queueCmdRecord.id / 6 == Robot.robot1.id && Robot.robot1.waitCount >= 2)
                        {
                            Thread.Sleep(10000);
                            continue;
                        }
                        if (queueCmdRecord.id / 6 == Robot.robot2.id && Robot.robot2.waitCount >= 2)
                        {
                            Thread.Sleep(10000);
                            continue;
                        }
                    }
                }

                if (initialized && EntityForCore.ins.globalStatus)
                {
                    Command queueCmd = null;
                    //等待系统总状态
                    Task getSysStatus = Task.Factory.StartNew(() =>
                    {
                        GetTotalStatus();
                    });
                    //等待号牌资源
                    Task getCmd = Task.Factory.StartNew(() =>
                    {
                        while (true)
                        {
                            queueCmd = (Command)queuingThread.GetMessage();
                            if (queueCmd != null)
                            {
                                //远端或本地异常，遇到相应指令则退回
                                if ((!queueCmd.manual && ConnectionPoolManager.malfunctionRemote) || (queueCmd.manual && ConnectionPoolManager.malfunctionLocal))
                                {
                                    queueCmd.returnedCount = 1;
                                    queuingThread.SetMessage((Command)queueCmd.Clone());
                                    Thread.Sleep(5000);
                                }
                                else
                                {
                                    break;
                                }
                            }
                        }
                    });
                    Task.WaitAll(getSysStatus, getCmd);


                    lock (lockObj)
                    {
                        if (queueCmdRecord != null)
                        {
                            //对依赖同一机械手的线程计数
                            if (queueCmdRecord.id == Robot.robot1.id)
                            {
                                Robot.robot1.waitCount += 1;
                            }
                            else if (queueCmdRecord.id == Robot.robot2.id)
                            {
                                Robot.robot2.waitCount += 1;
                            }
                        }
                        queueCmdRecord = (Command)queueCmd.Clone();
                    }
                    Log.WriteLog(LogType.NOT_DATABASE, LogFile.LOG, "线程创建:" + queueCmd.LicenseNum + "准备，等待机械手线程个数：" + Robot.robot1.waitCount);
                    Task.Factory.StartNew(() =>
                    {
                        Log.WriteLog(LogType.NOT_DATABASE, LogFile.LOG, "线程创建:" + queueCmd.LicenseNum + "开始");
                        SimpleCMDFactory simpleCMDFactory = new SimpleCMDFactory();
                        AbstractCmd abstractCmd = simpleCMDFactory.createCmd(queueCmd);
                        abstractCmd.executeCmd(queueCmd);
                        Log.WriteLog(LogType.NOT_DATABASE, LogFile.LOG, "线程创建:" + queueCmd.LicenseNum + "完成");
                    });

                }
                Thread.Sleep(100);
            }
        }

        /// <summary>挂起</summary>
        public void hangUp()
        {

        }
        /// <summary>
        /// 获取PLC总控状态
        /// </summary>
        private void GetTotalStatus()
        {
            PLCMessage PLCMsg = null;
            int normalStatus = 0;
            while (!isClosing)
            {
                PLCMsg = (PLCMessage)PLC.GetMessage();
                if (PLCMsg != null && PLCMsg.originalPlcList != null && PLCMsg.originalPlcList.Count != 0)
                {
                    normalStatus = Convert.ToInt32(PLCMsg.originalPlcList[equipmentStatus_address].Value);
                }
                //设备总控状态
                if (normalStatus == 1 && EntityForCore.ins.globalStatus && ParkingBufferManager.ins.checkedAuto)
                {
                    UILogServer.ins.log("设备总状态正常");
                    Log.WriteLog(LogType.NOT_DATABASE, LogFile.LOG, "设备总状态正常");
                    break;
                }
                Thread.Sleep(200);
            }
        }

        /// <summary>
        ///  执行日志文件数据库操作
        /// </summary>
        /// <param name="o"></param>
        private void displayLog(object o)
        {
            while (!EntityForCore.ins.globalStatus)
            {
                Thread.Sleep(1000);
            }
            lock (Parking_Space.spaceLock)
            {
                string sqlStatus;
                string sqlMsg;
                int count;
                Log.ReadLog(out sqlStatus, out sqlMsg, out count);
                List<string> strs = new List<string>();
                strs.Add(sqlMsg);
                if (count > 0)
                {
                    if (sqlStatus == "1")
                    {
                        int parkingRecordsID = 0;
                        try
                        {
                            //Operation.MyTransaction(EntityForCore.remoteBQ,sqlMsg, out parkingRecordsID);
                            parkingRecordsID = DBOperation.InsertParkingRecords(EntityForCore.remoteBQ, strs);
                            if (parkingRecordsID == 0)
                            {
                                Log.WriteLog(LogType.DATABASE, sqlStatus + ":" + sqlMsg);
                            }
                        }
                        catch
                        {
                            Log.WriteLog(LogType.DATABASE, sqlStatus + ":" + sqlMsg);
                            //throw;
                        }
                    }
                    if (sqlStatus == "0")
                    {
                        bool result;
                        try
                        {
                            //Operation.MyTransaction(EntityForCore.remoteBQ,sqlMsg, out temp); 
                            result = DBOperation.UpdateTransaction(EntityForCore.remoteBQ, strs);
                            if (!result)
                            {
                                Log.WriteLog(LogType.DATABASE, sqlStatus + ":" + sqlMsg);
                            }
                        }
                        catch
                        {
                            Log.WriteLog(LogType.DATABASE, sqlStatus + ":" + sqlMsg);
                            //throw;
                        }
                    }
                    Log.ReadLog(out sqlStatus, out sqlMsg, out count);
                }
            }
        }
        private void CheckRemotePool(object o)
        {
            ConnectionPoolManager.CheckConnPooling(3, 10, ConnectionPoolManager.remoteConf, EntityForCore.remoteBQ);
            DBOperation.KillTimeOutConnection(null, 3600, EntityForCore.remoteBQ);
        }
        private void CheckLocalPool(object o)
        {
            ConnectionPoolManager.CheckConnPooling(3, 10, ConnectionPoolManager.localConf, EntityForCore.localBQ);
        }
    }
}
